@@ -575,8 +575,6 @@ int CSndBuffer::readData(const int offset, CPacket& w_packet, steady_clock::time
     // the packet originally (the other overload of this function) must set these
     // flags.
     w_packet.m_iMsgNo = p->m_iMsgNoBitset;
-
-    // TODO: FR #930. Use source time if it is provided.
     w_srctime = getSourceTime(*p);
 
     // This function is called when packet retransmission is triggered.
@@ -935,7 +933,7 @@ int CRcvBuffer::readBuffer(char* data, int len)
                 break; /* too early for this unit, return whatever was copied */
         }
 
-        const int pktlen = pkt.getLength();
+        const int pktlen = (int) pkt.getLength();
         const int remain_pktlen = pktlen - m_iNotch;
 
         const int unitsize = std::min(remain_pktlen, rs);
@@ -996,7 +994,7 @@ int CRcvBuffer::readBufferToFile(fstream& ofs, int len)
 #if ENABLE_LOGGING
         trace_seq = pkt.getSeqNo();
 #endif
-        const int pktlen = pkt.getLength();
+        const int pktlen = (int) pkt.getLength();
         const int remain_pktlen = pktlen - m_iNotch;
 
         const int unitsize = std::min(remain_pktlen, rs);
@@ -1770,8 +1768,8 @@ steady_clock::time_point CRcvBuffer::getTsbPdTimeBase(uint32_t timestamp_us)
         {
             carryover = int64_t(CPacket::MAX_TIMESTAMP) + 1;
         }
-        //
-        else if ((timestamp_us >= TSBPD_WRAP_PERIOD) && (timestamp_us <= (TSBPD_WRAP_PERIOD * 2)))
+        // timestamp_us >= TSBPD_WRAP_PERIOD
+        else if (timestamp_us <= (TSBPD_WRAP_PERIOD * 2))
         {
             /* Exiting wrap check period (if for packet delivery head) */
             m_bTsbPdWrapCheck = false;
@@ -2066,15 +2064,15 @@ int CRcvBuffer::readMsg(char* data, int len, SRT_MSGCTRL& w_msgctl, int upto)
 }
 
 #ifdef SRT_DEBUG_TSBPD_OUTJITTER
-void CRcvBuffer::debugTraceJitter(int64_t rplaytime)
+void CRcvBuffer::debugTraceJitter(time_point playtime)
 {
-    uint64_t now = CTimer::getTime();
-    if ((now - rplaytime) / 10 < 10)
-        m_ulPdHisto[0][(now - rplaytime) / 10]++;
-    else if ((now - rplaytime) / 100 < 10)
-        m_ulPdHisto[1][(now - rplaytime) / 100]++;
-    else if ((now - rplaytime) / 1000 < 10)
-        m_ulPdHisto[2][(now - rplaytime) / 1000]++;
+    uint64_t ms = count_microseconds(steady_clock::now() - playtime);
+    if (ms / 10 < 10)
+        m_ulPdHisto[0][ms / 10]++;
+    else if (ms / 100 < 10)
+        m_ulPdHisto[1][ms / 100]++;
+    else if (ms / 1000 < 10)
+        m_ulPdHisto[2][ms / 1000]++;
     else
         m_ulPdHisto[3][1]++;
 }
@@ -2107,7 +2105,7 @@ bool CRcvBuffer::accessMsg(int& w_p, int& w_q, bool& w_passack, int64_t& w_playt
             // so in one "unit".
             w_p = w_q = m_iStartPos;
 
-            debugTraceJitter(w_playtime);
+            debugTraceJitter(play_time);
         }
     }
     else
@@ -2297,7 +2295,7 @@ bool CRcvBuffer::scanMsg(int& w_p, int& w_q, bool& w_passack)
         }
 
         rmpkts++;
-        rmbytes += freeUnitAt(m_iStartPos);
+        rmbytes += (int) freeUnitAt((size_t) m_iStartPos);
 
         m_iStartPos = shiftFwd(m_iStartPos);
     }
