@@ -72,6 +72,11 @@ modified by
 
 using namespace srt::sync;
 
+namespace srt_logging {
+    extern Logger inlog;
+}
+
+using namespace srt_logging;
 
 CUDTException::CUDTException(CodeMajor major, CodeMinor minor, int err):
 m_iMajor(major),
@@ -83,230 +88,18 @@ m_iMinor(minor)
       m_iErrno = err;
 }
 
-const char* CUDTException::getErrorMessage() const ATR_NOTHROW
-{
-    return getErrorString().c_str();
+namespace srt {
+const char* strerror_get_message(size_t major, size_t minor);
 }
 
-const string& CUDTException::getErrorString() const
+const char* CUDTException::getErrorMessage() const ATR_NOTHROW
 {
-   // translate "Major:Minor" code into text message.
+    return srt::strerror_get_message(m_iMajor, m_iMinor);
+}
 
-   switch (m_iMajor)
-   {
-      case MJ_SUCCESS:
-        m_strMsg = "Success";
-        break;
-
-      case MJ_SETUP:
-        m_strMsg = "Connection setup failure";
-
-        switch (m_iMinor)
-        {
-        case MN_TIMEOUT:
-           m_strMsg += ": connection time out";
-           break;
-
-        case MN_REJECTED:
-           m_strMsg += ": connection rejected";
-           break;
-
-        case MN_NORES:
-           m_strMsg += ": unable to create/configure SRT socket";
-           break;
-
-        case MN_SECURITY:
-           m_strMsg += ": abort for security reasons";
-           break;
-
-        case MN_CLOSED:
-           m_strMsg += ": socket closed during operation";
-           break;
-
-        default:
-           break;
-        }
-
-        break;
-
-      case MJ_CONNECTION:
-        switch (m_iMinor)
-        {
-        case MN_CONNLOST:
-           m_strMsg = "Connection was broken";
-           break;
-
-        case MN_NOCONN:
-           m_strMsg = "Connection does not exist";
-           break;
-
-        default:
-           break;
-        }
-
-        break;
-
-      case MJ_SYSTEMRES:
-        m_strMsg = "System resource failure";
-
-        switch (m_iMinor)
-        {
-        case MN_THREAD:
-           m_strMsg += ": unable to create new threads";
-           break;
-
-        case MN_MEMORY:
-           m_strMsg += ": unable to allocate buffers";
-           break;
-
-
-        case MN_OBJECT:
-           m_strMsg += ": unable to allocate system object";
-           break;
-
-        default:
-           break;
-        }
-
-        break;
-
-      case MJ_FILESYSTEM:
-        m_strMsg = "File system failure";
-
-        switch (m_iMinor)
-        {
-        case MN_SEEKGFAIL:
-           m_strMsg += ": cannot seek read position";
-           break;
-
-        case MN_READFAIL:
-           m_strMsg += ": failure in read";
-           break;
-
-        case MN_SEEKPFAIL:
-           m_strMsg += ": cannot seek write position";
-           break;
-
-        case MN_WRITEFAIL:
-           m_strMsg += ": failure in write";
-           break;
-
-        default:
-           break;
-        }
-
-        break;
-
-      case MJ_NOTSUP:
-        m_strMsg = "Operation not supported";
- 
-        switch (m_iMinor)
-        {
-        case MN_ISBOUND:
-           m_strMsg += ": Cannot do this operation on a BOUND socket";
-           break;
-
-        case MN_ISCONNECTED:
-           m_strMsg += ": Cannot do this operation on a CONNECTED socket";
-           break;
-
-        case MN_INVAL:
-           m_strMsg += ": Bad parameters";
-           break;
-
-        case MN_SIDINVAL:
-           m_strMsg += ": Invalid socket ID";
-           break;
-
-        case MN_ISUNBOUND:
-           m_strMsg += ": Cannot do this operation on an UNBOUND socket";
-           break;
-
-        case MN_NOLISTEN:
-           m_strMsg += ": Socket is not in listening state";
-           break;
-
-        case MN_ISRENDEZVOUS:
-           m_strMsg += ": Listen/accept is not supported in rendezous connection setup";
-           break;
-
-        case MN_ISRENDUNBOUND:
-           m_strMsg += ": Cannot call connect on UNBOUND socket in rendezvous connection setup";
-           break;
-
-        case MN_INVALMSGAPI:
-           m_strMsg += ": Incorrect use of Message API (sendmsg/recvmsg).";
-           break;
-
-        case MN_INVALBUFFERAPI:
-           m_strMsg += ": Incorrect use of Buffer API (send/recv) or File API (sendfile/recvfile).";
-           break;
-
-        case MN_BUSY:
-           m_strMsg += ": Another socket is already listening on the same port";
-           break;
-
-        case MN_XSIZE:
-           m_strMsg += ": Message is too large to send (it must be less than the SRT send buffer size)";
-           break;
-
-        case MN_EIDINVAL:
-           m_strMsg += ": Invalid epoll ID";
-           break;
-
-        case MN_EEMPTY:
-           m_strMsg += ": All sockets removed from epoll, waiting would deadlock";
-
-        default:
-           break;
-        }
-
-        break;
-
-     case MJ_AGAIN:
-        m_strMsg = "Non-blocking call failure";
-
-        switch (m_iMinor)
-        {
-        case MN_WRAVAIL:
-           m_strMsg += ": no buffer available for sending";
-           break;
-
-        case MN_RDAVAIL:
-           m_strMsg += ": no data available for reading";
-           break;
-
-        case MN_XMTIMEOUT:
-           m_strMsg += ": transmission timed out";
-           break;
-
-#ifdef SRT_ENABLE_ECN
-        case MN_CONGESTION:
-           m_strMsg += ": early congestion notification";
-           break;
-#endif /* SRT_ENABLE_ECN */
-        default:
-           break;
-        }
-
-        break;
-
-     case MJ_PEERERROR:
-        m_strMsg = "The peer side has signalled an error";
-
-        break;
-
-      default:
-        m_strMsg = "Unknown error";
-   }
-
-   // Adding "errno" information
-   if ((MJ_SUCCESS != m_iMajor) && (0 < m_iErrno))
-   {
-      m_strMsg += ": " + SysStrError(m_iErrno);
-   }
-
-   return m_strMsg;
+string CUDTException::getErrorString() const
+{
+    return getErrorMessage();
 }
 
 #define UDT_XCODE(mj, mn) (int(mj)*1000)+int(mn)
@@ -364,7 +157,10 @@ void CIPAddress::ntop(const sockaddr_any& addr, uint32_t ip[4])
 {
     if (addr.family() == AF_INET)
     {
+        // SRT internal format of IPv4 address.
+        // The IPv4 address is in the first field. The rest is 0.
         ip[0] = addr.sin.sin_addr.s_addr;
+        ip[1] = ip[2] = ip[3] = 0;
     }
     else
     {
@@ -376,92 +172,143 @@ void CIPAddress::ntop(const sockaddr_any& addr, uint32_t ip[4])
     }
 }
 
+bool checkMappedIPv4(const uint16_t* addr)
+{
+    static const uint16_t ipv4on6_model [8] =
+    {
+        0, 0, 0, 0, 0, 0xFFFF, 0, 0
+    };
+
+    // Compare only first 6 words. Remaining 2 words
+    // comprise the IPv4 address, if these first 6 match.
+    const uint16_t* mbegin = ipv4on6_model;
+    const uint16_t* mend = ipv4on6_model + 6;
+
+    return std::equal(mbegin, mend, addr);
+}
+
 // XXX This has void return and the first argument is passed by reference.
 // Consider simply returning sockaddr_any by value.
-void CIPAddress::pton(sockaddr_any& w_addr, const uint32_t ip[4], int agent_family, const sockaddr_any& peer)
+void CIPAddress::pton(sockaddr_any& w_addr, const uint32_t ip[4], const sockaddr_any& peer)
 {
-   if (agent_family == AF_INET)
-   {
-      sockaddr_in* a = (&w_addr.sin);
-      a->sin_addr.s_addr = ip[0];
-   }
-   else // AF_INET6
-   {
-      // Check if the peer address is a model of IPv4-mapped-on-IPv6.
-      // If so, it means that the `ip` array should be interpreted as IPv4.
+    uint32_t* target_ipv4_addr = NULL;
 
-      uint16_t* peeraddr16 = (uint16_t*)peer.sin6.sin6_addr.s6_addr;
-      static const uint16_t ipv4on6_model [8] =
-      {
-          0, 0, 0, 0, 0, 0xFFFF, 0, 0
-      };
+    if (peer.family() == AF_INET)
+    {
+        sockaddr_in* a = (&w_addr.sin);
+        target_ipv4_addr = (uint32_t*) &a->sin_addr.s_addr;
+    }
+    else // AF_INET6
+    {
+        // Check if the peer address is a model of IPv4-mapped-on-IPv6.
+        // If so, it means that the `ip` array should be interpreted as IPv4.
+        const bool is_mapped_ipv4 = checkMappedIPv4((uint16_t*)peer.sin6.sin6_addr.s6_addr);
 
-      // Compare only first 6 words. Remaining 2 words
-      // comprise the IPv4 address, if these first 6 match.
-      const uint16_t* mbegin = ipv4on6_model;
-      const uint16_t* mend = ipv4on6_model + 6;
+        sockaddr_in6* a = (&w_addr.sin6);
 
-      bool is_mapped_ipv4 = (std::mismatch(mbegin, mend, peeraddr16).first == mend);
+        // This whole above procedure was only in order to EXCLUDE the
+        // possibility of IPv4-mapped-on-IPv6. This below may only happen
+        // if BOTH peers are IPv6. Otherwise we have a situation of cross-IP
+        // version connection in which case the address in question is always
+        // IPv4 in various mapping formats.
+        if (!is_mapped_ipv4)
+        {
+            // Here both agent and peer use IPv6, in which case
+            // `ip` contains the full IPv6 address, so just copy
+            // it as is.
 
-      sockaddr_in6* a = (&w_addr.sin6);
+            // XXX Possibly, a simple
+            // memcpy( (a->sin6_addr.s6_addr), ip, 16);
+            // would do the same thing, and faster. The address in `ip`,
+            // even though coded here as uint32_t, is still big endian.
+            for (int i = 0; i < 4; ++ i)
+            {
+                a->sin6_addr.s6_addr[i * 4 + 0] = ip[i] & 0xFF;
+                a->sin6_addr.s6_addr[i * 4 + 1] = (unsigned char)((ip[i] & 0xFF00) >> 8);
+                a->sin6_addr.s6_addr[i * 4 + 2] = (unsigned char)((ip[i] & 0xFF0000) >> 16);
+                a->sin6_addr.s6_addr[i * 4 + 3] = (unsigned char)((ip[i] & 0xFF000000) >> 24);
+            }
+            return; // The address is written, nothing left to do.
+        }
 
-      if (!is_mapped_ipv4)
-      {
-          // Here both agent and peer use IPv6, in which case
-          // `ip` contains the full IPv6 address, so just copy
-          // it as is.
+        // 
+        // IPv4 mapped on IPv6
 
-          // XXX Possibly, a simple
-          // memcpy( (a->sin6_addr.s6_addr), ip, 16);
-          // would do the same thing, and faster. The address in `ip`,
-          // even though coded here as uint32_t, is still big endian.
-          for (int i = 0; i < 4; ++ i)
-          {
-              a->sin6_addr.s6_addr[i * 4 + 0] = ip[i] & 0xFF;
-              a->sin6_addr.s6_addr[i * 4 + 1] = (unsigned char)((ip[i] & 0xFF00) >> 8);
-              a->sin6_addr.s6_addr[i * 4 + 2] = (unsigned char)((ip[i] & 0xFF0000) >> 16);
-              a->sin6_addr.s6_addr[i * 4 + 3] = (unsigned char)((ip[i] & 0xFF000000) >> 24);
-          }
-      }
-      else // IPv4 mapped on IPv6
-      {
-          // Here agent uses IPv6 with IPPROTO_IPV6/IPV6_V6ONLY == 0
-          // In this case, the address in `ip` uses only the `ip[0]`
-          // element carrying the IPv4 address, rest of the elements
-          // is 0. This address must be interpreted as IPv4, but set
-          // as the IPv4-on-IPv6 address.
-          //
-          // Unfortunately, sockaddr_in6 doesn't give any straightforward
-          // method for it, although the official size of a single element
-          // of the IPv6 address is 16-bit.
+        // Here agent uses IPv6 with IPPROTO_IPV6/IPV6_V6ONLY == 0
+        // In this case, the address in `ip` is always an IPv4,
+        // although we are not certain as to whether it's using the
+        // IPv6 encoding (0::FFFF:IPv4) or SRT encoding (IPv4::0);
+        // this must be extra determined.
+        //
+        // Unfortunately, sockaddr_in6 doesn't give any straightforward
+        // method for it, although the official size of a single element
+        // of the IPv6 address is 16-bit.
 
-          memset((a->sin6_addr.s6_addr), 0, sizeof a->sin6_addr.s6_addr);
+        memset((a->sin6_addr.s6_addr), 0, sizeof a->sin6_addr.s6_addr);
 
-          // There are also available sin6_addr.s6_addr32, but
-          // this is kinda nonportable.
-          uint32_t* paddr32 = (uint32_t*)a->sin6_addr.s6_addr;
-          uint16_t* paddr16 = (uint16_t*)a->sin6_addr.s6_addr;
+        // The sin6_addr.s6_addr32 is non that portable to use here.
+        uint32_t* paddr32 = (uint32_t*)a->sin6_addr.s6_addr;
+        uint16_t* paddr16 = (uint16_t*)a->sin6_addr.s6_addr;
 
-          // layout: of IPv4 address 192.168.128.2
-          // 16-bit:
-          // [0000: 0000: 0000: 0000: 0000: FFFF: 192.168:128.2]
-          // 8-bit
-          // [00/00/00/00/00/00/00/00/00/00/FF/FF/192/168/128/2]
-          // 32-bit
-          // [00000000 && 00000000 && 0000FFFF && 192.168.128.2]
+        // layout: of IPv4 address 192.168.128.2
+        // 16-bit:
+        // [0000: 0000: 0000: 0000: 0000: FFFF: 192.168:128.2]
+        // 8-bit
+        // [00/00/00/00/00/00/00/00/00/00/FF/FF/192/168/128/2]
+        // 32-bit
+        // [00000000 && 00000000 && 0000FFFF && 192.168.128.2]
 
-          // Spreading every 16-bit word separately to avoid endian dilemmas
-          paddr16[2 * 2 + 0] = 0;
-          paddr16[2 * 2 + 1] = 0xFFFF;
+        // Spreading every 16-bit word separately to avoid endian dilemmas
+        paddr16[2 * 2 + 1] = 0xFFFF;
 
-          paddr32[3] = ip[0]; // IPv4 address encoded here
-      }
-   }
+        target_ipv4_addr = &paddr32[3];
+    }
+
+    // Now we have two possible formats of encoding the IPv4 address:
+    // 1. If peer is IPv4, it's IPv4::0
+    // 2. If peer is IPv6, it's 0::FFFF:IPv4.
+    //
+    // Has any other possibility happen here, copy an empty address,
+    // which will be the only sign of an error.
+
+    const uint16_t* peeraddr16 = (uint16_t*)ip;
+    const bool is_mapped_ipv4 = checkMappedIPv4(peeraddr16);
+
+    if (is_mapped_ipv4)
+    {
+        *target_ipv4_addr = ip[3];
+        HLOGC(inlog.Debug, log << "pton: Handshake address: " << w_addr.str() << " provided in IPv6 mapping format");
+    }
+    // Check SRT IPv4 format.
+    else if ((ip[1] | ip[2] | ip[3]) == 0)
+    {
+        *target_ipv4_addr = ip[0];
+        HLOGC(inlog.Debug, log << "pton: Handshake address: " << w_addr.str() << " provided in SRT IPv4 format");
+    }
+    else
+    {
+        LOGC(inlog.Error, log << "pton: IPE or net error: can't determine IPv4 carryover format: " << std::hex
+                << peeraddr16[0] << ":"
+                << peeraddr16[1] << ":"
+                << peeraddr16[2] << ":"
+                << peeraddr16[3] << ":"
+                << peeraddr16[4] << ":"
+                << peeraddr16[5] << ":"
+                << peeraddr16[6] << ":"
+                << peeraddr16[7] << std::dec);
+        *target_ipv4_addr = 0;
+        if (peer.family() != AF_INET)
+        {
+            // Additionally overwrite the 0xFFFF that has been
+            // just written 50 lines above.
+            w_addr.sin6.sin6_addr.s6_addr[10] = 0;
+            w_addr.sin6.sin6_addr.s6_addr[11] = 0;
+        }
+    }
 }
 
 using namespace std;
 
-
 static string ShowIP4(const sockaddr_in* sin)
 {
     ostringstream os;
@@ -520,7 +367,7 @@ void CMD5::compute(const char* input, unsigned char result[16])
    md5_state_t state;
 
    md5_init(&state);
-   md5_append(&state, (const md5_byte_t *)input, strlen(input));
+   md5_append(&state, (const md5_byte_t *)input, (int) strlen(input));
    md5_finish(&state, result);
 }
 
@@ -615,7 +462,7 @@ extern const char* const srt_rejectreason_msg [] = {
     "Password required or unexpected",
     "MessageAPI/StreamAPI collision",
     "Congestion controller type collision",
-    "Packet Filter type collision",
+    "Packet Filter settings error",
     "Group settings collision",
     "Connection timeout"
 };
@@ -648,7 +495,8 @@ bool SrtParseConfig(string s, SrtConfig& w_config)
         Split(*i, ':', back_inserter(keyval));
         if (keyval.size() != 2)
             return false;
-        w_config.parameters[keyval[0]] = keyval[1];
+        if (keyval[1] != "")
+            w_config.parameters[keyval[0]] = keyval[1];
     }
 
     return true;
@@ -661,12 +509,11 @@ uint64_t PacketMetric::fullBytes()
 }
 
 
-// Some logging imps
-#if ENABLE_LOGGING
-
 namespace srt_logging
 {
 
+// Value display utilities
+// (also useful for applications)
 
 std::string SockStatusStr(SRT_SOCKSTATUS s)
 {
@@ -722,6 +569,10 @@ std::string MemberStatusStr(SRT_MEMBERSTATUS s)
 }
 #endif
 
+// Logging system implementation
+
+#if ENABLE_LOGGING
+
 LogDispatcher::Proxy::Proxy(LogDispatcher& guy) : that(guy), that_enabled(that.CheckEnabled())
 {
     if (that_enabled)
@@ -834,7 +685,7 @@ std::string LogDispatcher::Proxy::ExtractName(std::string pretty_function)
 
     return pretty_function.substr(pos+2);
 }
+#endif
 
 } // (end namespace srt_logging)
 
-#endif
