@@ -60,7 +60,7 @@ modified by
 #include "udt.h"
 
 
-struct CEPollDesc
+class CEPollDesc
 {
    const int m_iID;                                // epoll ID
 
@@ -143,8 +143,6 @@ struct CEPollDesc
 std::string DisplayEpollWatch();
 #endif
 
-private:
-
    /// Sockets that are subscribed for events in this eid.
    ewatch_t m_USockWatchState;
 
@@ -159,7 +157,10 @@ private:
 
    enotice_t::iterator nullNotice() { return m_USockEventNotice.end(); }
 
-public:
+   // Only CEPoll class should have access to it.
+   // Guarding private access to the class is not necessary
+   // within the epoll module.
+   friend class CEPoll;
 
    CEPollDesc(int id, int localID)
        : m_iID(id)
@@ -423,6 +424,9 @@ public: // for CUDTUnited API
    /// @retval >=0 number of ready sockets (actually size of `st`)
    int swait(CEPollDesc& d, fmap_t& st, int64_t msTimeOut, bool report_by_exception = true);
 
+   /// Empty subscription check - for internal use only.
+   bool empty(const CEPollDesc& d) const;
+
    /// Reports which events are ready on the given socket.
    /// @param mp socket event map retirned by `swait`
    /// @param sock which socket to ask
@@ -465,12 +469,13 @@ public: // for CUDTUnited API
 
 public: // for CUDT to acknowledge IO status
 
-   /// Update events available for a UDT socket.
+   /// Update events available for a UDT socket. At the end this function
+   /// counts the number of updated EIDs with given events.
    /// @param [in] uid UDT socket ID.
    /// @param [in] eids EPoll IDs to be set
    /// @param [in] events Combination of events to update
    /// @param [in] enable true -> enable, otherwise disable
-   /// @return 0 if success, otherwise an error number
+   /// @return -1 if invalid events, otherwise the number of changes
 
    int update_events(const SRTSOCKET& uid, std::set<int>& eids, int events, bool enable);
 
@@ -481,7 +486,7 @@ private:
    srt::sync::Mutex m_SeedLock;
 
    std::map<int, CEPollDesc> m_mPolls;       // all epolls
-   srt::sync::Mutex m_EPollLock;
+   mutable srt::sync::Mutex m_EPollLock;
 };
 
 #if ENABLE_HEAVY_LOGGING
