@@ -54,9 +54,9 @@ modified by
 #ifndef INC_SRT_QUEUE_H
 #define INC_SRT_QUEUE_H
 
-#include "channel.h"
 #include "common.h"
 #include "packet.h"
+#include "socketconfig.h"
 #include "netinet_any.h"
 #include "utilities.h"
 #include <list>
@@ -65,6 +65,7 @@ modified by
 #include <vector>
 
 class CUDT;
+class CChannel;
 
 struct CUnit
 {
@@ -329,11 +330,7 @@ public:
    void insert(const SRTSOCKET& id, CUDT* u, const sockaddr_any& addr,
                const srt::sync::steady_clock::time_point &ttl);
 
-   // The should_lock parameter is given here to state as to whether
-   // the lock should be applied here. If called from some internals
-   // and the lock IS ALREADY APPLIED, use false here to prevent
-   // double locking and deadlock in result.
-   void remove(const SRTSOCKET& id, bool should_lock);
+   void remove(const SRTSOCKET& id);
    CUDT* retrieve(const sockaddr_any& addr, SRTSOCKET& id);
 
    void updateConnStatus(EReadStatus rst, EConnectStatus, const CPacket& response);
@@ -396,8 +393,8 @@ public:
    bool getBind(char* dst, size_t len) const;
 #endif
 
-   int ioctlQuery(int type) const { return m_pChannel->ioctlQuery(type); }
-   int sockoptQuery(int level, int type) const { return m_pChannel->sockoptQuery(level, type); }
+   int ioctlQuery(int type) const;
+   int sockoptQuery(int level, int type) const;
 
    void setClosing()
    {
@@ -465,7 +462,7 @@ public:
       /// @param [in] c UDP channel to be associated to the queue
       /// @param [in] t timer
 
-   void init(int size, int payload, int version, int hsize, CChannel* c, srt::sync::CTimer* t);
+   void init(int size, size_t payload, int version, int hsize, CChannel* c, srt::sync::CTimer* t);
 
       /// Read a packet for a specific UDT socket id.
       /// @param [in] id Socket ID
@@ -497,7 +494,7 @@ private:
    CChannel* m_pChannel;        // UDP channel for receving packets
    srt::sync::CTimer* m_pTimer; // shared timer with the snd queue
 
-   int m_iPayloadSize;          // packet payload size
+   size_t m_szPayloadSize;      // packet payload size
 
    volatile bool m_bClosing;    // closing the worker
 #if ENABLE_LOGGING
@@ -509,7 +506,7 @@ private:
    void removeListener(const CUDT* u);
 
    void registerConnector(const SRTSOCKET& id, CUDT* u, const sockaddr_any& addr, const srt::sync::steady_clock::time_point& ttl);
-   void removeConnector(const SRTSOCKET& id, bool should_lock = true);
+   void removeConnector(const SRTSOCKET& id);
 
    void setNewEntry(CUDT* u);
    bool ifNewEntry();
@@ -543,15 +540,9 @@ struct CMultiplexer
 
    int m_iPort;         // The UDP port number of this multiplexer
    int m_iIPversion;    // Address family (AF_INET or AF_INET6)
-   int m_iIpTTL;
-   int m_iIpToS;
-#ifdef SRT_ENABLE_BINDTODEVICE
-   std::string m_BindToDevice;
-#endif
-   int m_iMSS;          // Maximum Segment Size
    int m_iRefCount;     // number of UDT instances that are associated with this multiplexer
-   int m_iIpV6Only;     // IPV6_V6ONLY option
-   bool m_bReusable;    // if this one can be shared with others
+
+   CSrtMuxerConfig m_mcfg;
 
    int m_iID;           // multiplexer ID
 
